package ru.ulstu.extractor.ts.creator.scheduled;

import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.gitrepository.service.GitApi;
import ru.ulstu.extractor.gitrepository.service.GitAtheneApi;
import ru.ulstu.extractor.gitrepository.service.GithubApi;
import ru.ulstu.extractor.gitrepository.service.GitlabApi;
import ru.ulstu.extractor.ts.creator.AbstractTimeSeriesCreator;
import ru.ulstu.extractor.ts.model.TimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;
import ru.ulstu.extractor.util.StringUtils;

import java.util.Collections;
import java.util.Map;

public abstract class ScheduledTimeSeriesCreator extends AbstractTimeSeriesCreator {
    private final Map<String, Class<? extends GitApi>> gitApiServiceHolder = Map.of(
            "git.athene.tech", GitAtheneApi.class,
            "github.com", GithubApi.class,
            "gitlab.com", GitlabApi.class);

    @Transactional
    public void addTimeSeriesValue(Branch branch) {
        TimeSeries timeSeries = getTimeSeriesService().findByBranchAndTypeOrCreate(branch, getTimeSeriesType());
        timeSeries.getValues().add(getNewTimeSeriesValue(branch));
        getTimeSeriesService().save(Collections.singletonList(timeSeries), branch);
    }

    public abstract TimeSeriesValue getNewTimeSeriesValue(Branch branch);

    public Class<? extends GitApi> getGitApiServiceClass(Branch branch) {
        return gitApiServiceHolder.get(StringUtils.getServerDomain(branch.getGitRepository().getUrl()));
    }
}
